/*
    Copyright (c) 2002, 2003, bDistributed.com, Inc.
    All rights reserved.

    Redistribution and use in source and binary forms, with or
    without modification, are permitted provided that the following
    conditions are met:

    *   Redistributions of source code must retain the above
        copyright notice, this list of conditions and the following
        disclaimer.
    
    *   Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.
    
    *   Neither the name of bDistributed.com, Inc. nor the names of
        its contributors may be used to endorse or promote products
        derived from this software without specific prior written
        permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
    CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
    OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
    OF SUCH DAMAGE.
*/


#import "BDRule.h"
#import "BDAssignment.h"
#import "BDRuleContext.h"


@implementation BDRule

#pragma mark -
#pragma mark class methods

+ (id)ruleWithLhs:(NSPredicate *)inLhs
              rhs:(BDAssignment *)inRhs
         priority:(NSInteger)inPriority
{
    return [[self alloc] initWithLhs:inLhs
                                  rhs:inRhs
                             priority:inPriority];
}

#pragma mark -
#pragma mark init methods

- (id)initWithLhs:(NSPredicate *)inLhs
              rhs:(BDAssignment *)inRhs
         priority:(NSInteger)inPriority
{
    if (self = [super init])
    {
        self.lhs = inLhs;
        self.rhs = inRhs;
        self.priority = inPriority;
    }

    return self;
}

#pragma -
#pragma mark dealloc

- (void)dealloc
{
    self.lhs = nil;
    self.rhs = nil;
}

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

- (void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeObject:self.lhs];
    [coder encodeObject:self.rhs];
    [coder encodeObject:[NSNumber numberWithInteger:self.priority]];
}

- (id)initWithCoder:(NSCoder *)coder
{
    return [self initWithLhs:[coder decodeObject]
                         rhs:[coder decodeObject]
                    priority:[[coder decodeObject] integerValue]];
}

#pragma mark -
#pragma mark BDPropertyListEncoding methods
/* BDPropertyListEncoding Protocol */

- (id)initWithPropertyList:(NSDictionary *)propertyList
                     owner:(id)owner
{
    int version;
    NSPredicate *theLhs = nil;
    BDAssignment *theRhs = nil;
    NSInteger thePriority = 0;
    
    version = [[propertyList valueForKey:@"version"] intValue];
    
    if (version >= 1) {
        NSString *rhsClassName;
        Class rhsClass;
        
        // Decode the left-hand side (a qualifier) from a string
        
        theLhs = [NSPredicate predicateWithFormat:[propertyList valueForKey:@"lhs"]];
        
        // Decode the right-hand side class name
        
        rhsClassName = [propertyList valueForKey:@"assignmentClass"];
        
        rhsClass = NSClassFromString(rhsClassName);
        
        // Decode the right-hand side (an assignment)
        
        theRhs = [[rhsClass alloc] initWithPropertyList:[propertyList valueForKey:@"rhs"]
                                                   owner:self];
        
        // Decode the priority
        
        thePriority = [[propertyList valueForKey:@"priority"] integerValue];
    }
    
    return [self initWithLhs:theLhs
                         rhs:theRhs
                    priority:thePriority];
}

- (void)awakeWithPropertyList:(NSDictionary *)propertyList
{
    int version;
    
    version = [[propertyList valueForKey:@"version"] intValue];
    
    if (version >= 1) {
        [self.rhs awakeWithPropertyList:[propertyList valueForKey:@"rhs"]];
    }
}

- (void)encodeIntoPropertyList:(NSMutableDictionary *)propertyList
{
    NSMutableDictionary *plistAssignment;
    
    // Encode a version number
    
    [propertyList setValue:[NSNumber numberWithInt:1]
                    forKey:@"version"];
    
    // Encode the left-hand side (a qualifier) as a string
    
    [propertyList setValue:[self.lhs description]
                    forKey:@"lhs"];
    
    // Encode the right-hand side class name
    
    [propertyList setValue:NSStringFromClass([self.rhs class])
                    forKey:@"assignmentClass"];
    
    // Encode the right-hand side (an assignment)
    
    plistAssignment = [NSMutableDictionary dictionaryWithCapacity:0];
    
    [self.rhs encodeIntoPropertyList:plistAssignment];
    
    [propertyList setValue:plistAssignment
                    forKey:@"rhs"];
    
    // Encode the priority
    
    [propertyList setValue:[NSNumber numberWithInteger:self.priority]
                    forKey:@"priority"];
}

#pragma mark -
#pragma mark awakeFromNib-like methods

#pragma mark -
#pragma mark action methods

#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

@synthesize lhs;
@synthesize rhs;
@synthesize priority;

#pragma mark -
#pragma mark Utility methods

- (id)fireInContext:(BDRuleContext *)context
{
    return [rhs fireInContext:context];
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"(%@) => %@ [%ld]",
        [self.lhs description],
        [self.rhs description],
        (long)self.priority];
}

@end
