/*
    Copyright (c) 2002, 2003, bDistributed.com, Inc.
    All rights reserved.

    Redistribution and use in source and binary forms, with or
    without modification, are permitted provided that the following
    conditions are met:

    *   Redistributions of source code must retain the above
        copyright notice, this list of conditions and the following
        disclaimer.
    
    *   Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.
    
    *   Neither the name of bDistributed.com, Inc. nor the names of
        its contributors may be used to endorse or promote products
        derived from this software without specific prior written
        permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
    CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
    OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
    OF SUCH DAMAGE.
*/


#import "BDRuleContext.h"
#import "BDRule.h"
#import "BDRuleModel.h"

@implementation BDRuleContext

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

- (id)init
{
    return [self initWithModel:nil];
}

- (id)initWithModel:(BDRuleModel *)inModel
{
    if (self = [super init])
    {
        self.model = inModel;
        storedValues = [[NSMutableDictionary alloc] initWithCapacity:0];
    }

    return self;
}

#pragma -
#pragma mark dealloc

- (void)dealloc
{
    self.model = nil;
    storedValues = nil;
}

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark BDPropertyListEncoding methods

#pragma mark -
#pragma mark awakeFromNib-like methods

#pragma mark -
#pragma mark action methods

#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

@synthesize model;

#pragma mark -
#pragma mark Utility methods


// getter Helper
- (NSInteger) integerForKey:(NSString*)keyName
{
    NSNumber* theResult;
    
    theResult = [self valueForKey:keyName];
    
    NSAssert([theResult respondsToSelector:@selector(integerValue)], @"keyName is Not responds integerValue.");
    
    return [theResult integerValue];
}

- (NSUInteger) unsignedIntegerForKey:(NSString*)keyName
{
    NSNumber* theResult;
    
    theResult = [self valueForKey:keyName];
    
    NSAssert([theResult respondsToSelector:@selector(unsignedIntegerValue)], @"keyName is Not responds unsignedIntegerValue.");
    
    return [theResult unsignedIntegerValue];
}

- (float) floatForKey:(NSString*)keyName
{
    NSNumber* theResult;
    
    theResult = [self valueForKey:keyName];
    
    NSAssert([theResult respondsToSelector:@selector(floatValue)], @"keyName is Not responds floatValue.");
    
    return [theResult floatValue];
}

- (double) doubleForKey:(NSString*)keyName
{
    NSNumber* theResult;
    
    theResult = [self valueForKey:keyName];
    
    NSAssert([theResult respondsToSelector:@selector(doubleValue)], @"keyName is Not responds doubleValue.");
    
    return [theResult doubleValue];
}

- (BOOL) boolForKey:(NSString*)keyName
{
    NSNumber* theResult;
    
    theResult = [self valueForKey:keyName];
    
    NSAssert([theResult respondsToSelector:@selector(boolValue)], @"keyName is Not responds boolValue.");
    
    return [theResult boolValue];
}

// setter Helper
- (void)setInteger:(NSInteger)value forKey:(NSString*)keyName
{    
    [self setValue:[NSNumber numberWithInteger:value]
            forKey:keyName];
}

- (void)setUnsignedIntegerForKey:(NSUInteger)value forKey:(NSString*)keyName
{
    [self setValue:[NSNumber numberWithUnsignedInteger:value]
            forKey:keyName];
}

- (void)setFloat:(float)value forKey:(NSString *)keyName
{
    [self setValue:[NSNumber numberWithFloat:value]
            forKey:keyName];
}

- (void)setDouble:(double)value forKey:(NSString*)keyName
{
    [self setValue:[NSNumber numberWithDouble:value]
            forKey:keyName];
}

- (void)setBool:(BOOL)value forKey:(NSString*)keyName
{
    [self setValue:[NSNumber numberWithBool:value]
            forKey:keyName];
}


#pragma mark -
#pragma mark KVC

- (id)valueForKey:(NSString *)key
{
    id value;

    value = [self storedValueForKey:key];

    if (value == nil)
    {
        value = [self inferredValueForKey:key];
    }

    return value;
}

- (void)setValue:(id)value forKey:(NSString *)key
{
    [self takeStoredValue:value forKey:key];
}

- (id)storedValueForKey:(NSString *)key
{
    return [storedValues objectForKey:key];
}

- (void)takeStoredValue:(id)value forKey:(NSString *)key
{
    if (value != nil)
    {
        [storedValues setObject:value forKey:key];
    }
    else
    {
        [storedValues removeObjectForKey:key];
    }
}

- (id)inferredValueForKey:(NSString *)key
{
    id value = nil;
    NSArray *candidates;

    // Get all of the candidate rules for this key from our model.
 
    candidates = [[self model] candidateRulesForKey:key];

    if (candidates != nil)
    {
        BDRule *candidateRule;

        // Iterate through all of the candidate rules, evaluating their
        // left-hand side in this context.  If one returns YES, fire
        // it and return the value of firing it.  If none returns YES,
        // we just wind up returning nil.

        for (candidateRule in candidates)
        {
            if ([[candidateRule lhs] evaluateWithObject:self])
            {
                value = [candidateRule fireInContext:self];
                break;
            }
        }
    }

    return value;
}

- (NSArray *)valuesForKeyPath:(NSString *)keyPath
       takingSuccessiveValues:(NSArray *)values
                   forKeyPath:(NSString *)valueKeyPath
{
    NSMutableArray *result = [NSMutableArray arrayWithCapacity:[values count]];
    id oneValue;

    for (oneValue in values)
    {
        [self setValue:oneValue forKeyPath:valueKeyPath];
        [result addObject:[self valueForKeyPath:keyPath]];
    }

    return result;
}
@end
